using UnityEditor;
using UnityEngine;

namespace Sos
{
    // This attribute ensures the static constructor is called when Unity loads
    [InitializeOnLoad]
    public class AutoAddLayerOnLoad
    {
        static AutoAddLayerOnLoad()
        {
            AddLayer("PolygonStreaming"); // Specify the name of the layer you want to add
        }

        public static void AddLayer(string layerName)
        {
            // Check if the layer already exists
            if (!DoesLayerExist(layerName))
            {
                SerializedObject tagManager = new SerializedObject(AssetDatabase.LoadAllAssetsAtPath("ProjectSettings/TagManager.asset")[0]);
                SerializedProperty layersProp = tagManager.FindProperty("layers");

                for (int i = 8; i < layersProp.arraySize; i++) // Unity reserves layers 0-7
                {
                    SerializedProperty layerSP = layersProp.GetArrayElementAtIndex(i);
                    if (string.IsNullOrEmpty(layerSP.stringValue))
                    {
                        layerSP.stringValue = layerName;
                        tagManager.ApplyModifiedProperties();
                        //UnityEngine.Debug.Log($"Layer '{layerName}' has been added.");
                        return;
                    }
                }

                Debug.LogWarning("All layer slots are in use. Unable to add a new layer. Occlusion Culling will not function for Polygon Streaming.");
            }
            else
            {
                //Debug.Log($"Layer '{layerName}' already exists.");
            }
        }

        private static bool DoesLayerExist(string layerName)
        {
            for (int i = 0; i < 32; i++)
            {
                if (LayerMask.LayerToName(i) == layerName)
                {
                    return true;
                }
            }
            return false;
        }
    }

}
