#if UNITY_EDITOR
using Sos;
using Unity.CodeEditor;
using UnityEditor;
using UnityEngine;

[CustomEditor(typeof(StreamController))]
public class StreamControllerEditor : Editor
{
    private SerializedProperty viewerCameraProp;
    private SerializedProperty triangleBudgetProp;
#if SOS_EXACT_TREE_OPTION
    private SerializedProperty useExactTreeProp;
    private SerializedProperty desiredLevelProp;
    private SerializedProperty desiredLodProp;
#endif
    private SerializedProperty distanceFactorProp;
    private SerializedProperty clsoeUpDistanceProp;
    private SerializedProperty clsoeUpDistanceFactorProp;
    //private SerializedProperty distanceToProp;
    private SerializedProperty maxDownloadSizeMbProp;
    private SerializedProperty maximumQualityProp;

    private SerializedProperty occlusionCullingProp;
    private SerializedProperty raycastMaskProp;
    private SerializedProperty timeSlicingProp;
    private SerializedProperty rayPerFrameProp;

    private SerializedProperty streamControllerInitOptionsProp;
    private SerializedProperty streamControllerMonobehaviourConfigurationProp;

    private void OnEnable()
    {
        CodeEditor.Editor.CurrentCodeEditor.SyncAll();

        streamControllerInitOptionsProp = serializedObject.FindProperty(nameof(StreamController.streamControllerInitOptions));

        triangleBudgetProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.TriangleBudget));
#if SOS_EXACT_TREE_OPTION
        useExactTreeProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.UseExactTree));
        desiredLevelProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.DesiredLevel));
        desiredLodProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.DesiredLod));
#endif
        distanceFactorProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.DistanceFactor));
        clsoeUpDistanceProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.CloseUpDistance));
        clsoeUpDistanceFactorProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.CloseUpDistanceFactor));
        //distanceToProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.DistanceTo));
        maxDownloadSizeMbProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.MaxDownloadSizeInMB));
        maximumQualityProp = streamControllerInitOptionsProp.FindPropertyRelative(nameof(StreamControllerImplementation.StreamControllerInitOptions.MaximumQuality));

        streamControllerMonobehaviourConfigurationProp = serializedObject.FindProperty(nameof(StreamController.streamControllerMonobehaviourConfiguration));
        viewerCameraProp = streamControllerMonobehaviourConfigurationProp.FindPropertyRelative(nameof(StreamController.StreamControllerMonobehaviourConfiguration.viewerCamera));

        occlusionCullingProp = streamControllerMonobehaviourConfigurationProp.FindPropertyRelative(nameof(StreamController.StreamControllerMonobehaviourConfiguration.occlusionCulling));

        raycastMaskProp = streamControllerMonobehaviourConfigurationProp.FindPropertyRelative(nameof(StreamController.StreamControllerMonobehaviourConfiguration.raycastMask));
        timeSlicingProp = streamControllerMonobehaviourConfigurationProp.FindPropertyRelative(nameof(StreamController.StreamControllerMonobehaviourConfiguration.timeSlicing));
        rayPerFrameProp = streamControllerMonobehaviourConfigurationProp.FindPropertyRelative(nameof(StreamController.StreamControllerMonobehaviourConfiguration.raysPerFrame));
    }

    public override void OnInspectorGUI()
    {
        serializedObject.Update();

        EditorGUI.BeginChangeCheck();

        EditorGUILayout.PropertyField(viewerCameraProp);
        EditorGUILayout.PropertyField(triangleBudgetProp);
#if SOS_EXACT_TREE_OPTION
        EditorGUILayout.Space();
        if (useExactTreeProp.boolValue)
        {
            EditorGUILayout.PropertyField(useExactTreeProp);
            EditorGUILayout.PropertyField(desiredLevelProp);
            EditorGUILayout.PropertyField(desiredLodProp);
        }
        else
        {
            EditorGUILayout.PropertyField(useExactTreeProp);
        }
        EditorGUILayout.Space();
#endif

        EditorGUILayout.PropertyField(distanceFactorProp);
        EditorGUILayout.PropertyField(clsoeUpDistanceProp);
        EditorGUILayout.PropertyField(clsoeUpDistanceFactorProp);
        //EditorGUILayout.PropertyField(distanceToProp);
        EditorGUILayout.PropertyField(maxDownloadSizeMbProp);
        EditorGUILayout.PropertyField(maximumQualityProp);

        if (occlusionCullingProp.boolValue)
        {
            EditorGUI.BeginDisabledGroup(Application.isPlaying);
            EditorGUILayout.PropertyField(occlusionCullingProp);
            EditorGUILayout.PropertyField(raycastMaskProp);
            EditorGUILayout.PropertyField(timeSlicingProp);
            EditorGUILayout.PropertyField(rayPerFrameProp);
            EditorGUI.EndDisabledGroup();
        }
        else
        {
            EditorGUI.BeginDisabledGroup(Application.isPlaying);
            EditorGUILayout.PropertyField(occlusionCullingProp);
            EditorGUI.EndDisabledGroup();
        }
        serializedObject.ApplyModifiedProperties();
    }

}
#endif