#if UNITY_EDITOR
using Unity.CodeEditor;
using UnityEditor;

[CanEditMultipleObjects]
[CustomEditor(typeof(StreamingModel), true)]
public class StreamModelEditor : Editor
{
    private SerializedProperty streamControllerProp;
    private SerializedProperty sourceUrlProp;
    private SerializedProperty hashCodeProp;
    private SerializedProperty normalsProp;
    private SerializedProperty lightProbeUsageProp;
    private SerializedProperty forceMaterialsProp;
    private SerializedProperty materialProp;
    private SerializedProperty transparentMaterialProp;
    private SerializedProperty StreamingRuntimeOptionsSettingsProp;

    private void OnEnable()
    {
        CodeEditor.Editor.CurrentCodeEditor.SyncAll();

        streamControllerProp = serializedObject.FindProperty(nameof(StreamingModel.streamController));
        sourceUrlProp = serializedObject.FindProperty(nameof(StreamingModel.sourceUrl));
        hashCodeProp = serializedObject.FindProperty(nameof(StreamingModel.hashCode));
        normalsProp = serializedObject.FindProperty(nameof(StreamingModel.normals));
        lightProbeUsageProp = serializedObject.FindProperty(nameof(StreamingModel.lightProbeUsage));
        forceMaterialsProp = serializedObject.FindProperty(nameof(StreamingModel.customMaterials));
        materialProp = serializedObject.FindProperty(nameof(StreamingModel.material));
        transparentMaterialProp = serializedObject.FindProperty(nameof(StreamingModel.transparentMaterial));
        StreamingRuntimeOptionsSettingsProp = serializedObject.FindProperty(nameof(StreamingModel.StreamingRuntimeOptionsSettings));
    }

    public override void OnInspectorGUI()
    {
        serializedObject.Update();

        EditorGUI.BeginChangeCheck();

        EditorGUILayout.PropertyField(streamControllerProp);
        EditorGUILayout.PropertyField(sourceUrlProp);
        EditorGUILayout.PropertyField(hashCodeProp);
        EditorGUILayout.PropertyField(normalsProp);
        EditorGUILayout.PropertyField(lightProbeUsageProp);
        EditorGUILayout.PropertyField(forceMaterialsProp);
        EditorGUILayout.Space();

        if (forceMaterialsProp.boolValue)
        {
            EditorGUILayout.PropertyField(materialProp);
            EditorGUILayout.PropertyField(transparentMaterialProp);
        }

        EditorGUILayout.Space();
        EditorGUILayout.PropertyField(StreamingRuntimeOptionsSettingsProp);

        if (EditorGUI.EndChangeCheck())
        {
            serializedObject.ApplyModifiedProperties();
        }
    }
}
#endif
