var SOS_PLUGIN = {

  $BufferStorage : {
    value : null
  },

  $ModelStorage : {
    value : null
  },
  
  $Workers : {
	worker : null,
	handle : 0,
	initializing : false
  },

  $InternalFunctions : {
      getAplusArray : function (a) {
        return [a, a+1, a+2];
      },

      getPositions : function() {
        return ModelStorage.value["posBuff"];
      },

      getUVs : function(i) {
        return ModelStorage.value["uvBuff"][i];
      },

      getIndicies : function() {
        return ModelStorage.value["indBuff"];
      },

      getNormals : function() {
        return ModelStorage.value["norBuff"];
      }
  },

  $AdditionalFunctions : {
      getUvsSize : function() {
        return ModelStorage.value["uvBuff"].length;
      },

      getAlbedoX : function() {
        return ModelStorage.value["albedo"][0];
      },

      getAlbedoY : function() {
        return ModelStorage.value["albedo"][1];
      },

      getAlbedoZ : function() {
        return ModelStorage.value["albedo"][2];
      },

      getAlbedoW : function() {
        return ModelStorage.value["albedo"][3];
      },

      getEmissiveX : function() {
        return ModelStorage.value["emissiveScale"][0];
      },

      getEmissiveY : function() {
        return ModelStorage.value["emissiveScale"][1];
      },

      getEmissiveZ : function() {
        return ModelStorage.value["emissiveScale"][2];
      },

      getMetalness : function() {
        return ModelStorage.value["metalness"];
      },

      getRoughness : function() {
        return ModelStorage.value["roughness"];
      },

      getNormalScale : function() {
        return ModelStorage.value["normalScale"];
      },

      getOcclusionScale : function() {
        return ModelStorage.value["occlusionScale"];
      },

      getAlphaMode : function() {
        return ModelStorage.value["alphaMode"];
      }
  },

  SOSLoadGeometry: function (filenameBuf) {
	var filename = UTF8ToString(filenameBuf);
	
	var doDecompression = function(){
		var handle = Workers.handle;
		Workers.handle++;
		var byteArray;
		geomRequest = new XMLHttpRequest();
		geomRequest.open('GET', filename);
		geomRequest.responseType = 'arraybuffer';

		geomRequest.onload = function(ee)
		{ 
		  byteArray = new Uint8Array(this.response); 
		  var listener = function(event)  {
			if(event["data"]["handle"] == handle) {
			  ModelStorage.value = event["data"]["geom"];
			  Workers.worker.removeEventListener('message', listener);
			}
		  }

		  Workers.worker.addEventListener('message', listener);

		  Workers.worker.postMessage({
			"geometryArray": byteArray,
			"info": {
				"minX" : 0,
				"minY" : 0,
				"minZ" : 0,
				"maxX" : 1,
				"maxY" : 1,
				"maxZ" : 1
			  },
			"handle": handle
		  }, [byteArray.buffer]);
		  
		}
		geomRequest.send(null);		
	};
	
	if (!Workers.worker && !Workers.initializing) {
		Workers.initializing = true;
		const xrgRequest = new XMLHttpRequest();
		xrgRequest.open('GET', 'https://assets2.senseofspace.io/streamablemodel/0.0.12/xrgDecompressionWorker.js');  // `false` makes the request synchronous
		xrgRequest.responseType = 'blob';

		xrgRequest.onload = function(e)
		{
			Workers.worker = new Worker(URL.createObjectURL(this.response)); // Note:not xhr.responseText
			Workers.initializing = false;
			doDecompression()
		}
		xrgRequest.send(null);
	} else {
		var tryDoDecompression = function() {
			if (!Workers.worker && Workers.initializing){
				setTimeout( function() {
					tryDoDecompression();
				}, 500);				
				return;
			}
			doDecompression();
		}
		tryDoDecompression();
	}
  },

  SOSDecompressGeometry: function (byteBuff, length, id, decompressionCallback) {

    var byteArray = new Uint8Array(HEAPU8.buffer, byteBuff, length).slice();
	
	var doDecompression = function() {
		var handle = Workers.handle;
		Workers.handle++;

		  var listener = function(event) {
			if(event["data"]["handle"] == handle) {
			  ModelStorage.value = event["data"]["geom"];
			  dynCall_vi(decompressionCallback, id);
			  Workers.worker.removeEventListener('message', listener);
			} 
		  }

		  Workers.worker.addEventListener('message', listener);

		  Workers.worker.postMessage({
			"geometryArray": byteArray,
			"info": {
				"minX" : 0,
				"minY" : 0,
				"minZ" : 0,
				"maxX" : 1,
				"maxY" : 1,
				"maxZ" : 1
			  },
			"handle": handle
		  }, [byteArray.buffer]);		
	};
	
	if (!Workers.worker && !Workers.initializing) {
		Workers.initializing = true;
		const xrgRequest = new XMLHttpRequest();
		xrgRequest.open('GET', 'https://assets2.senseofspace.io/streamablemodel/0.0.12/xrgDecompressionWorker.js');  // `false` makes the request synchronous
		xrgRequest.responseType = 'blob';
		
		
		xrgRequest.onload = function(e)
		{
			Workers.worker = new Worker(URL.createObjectURL(this.response)); // Note:not xhr.responseText
			Workers.initializing = false;

			doDecompression();
		}
		xrgRequest.send(null);		
	} else {
		var tryDoDecompression = function() {
			if (!Workers.worker && Workers.initializing){
				setTimeout( function() {
					tryDoDecompression();
				}, 500);				
				return;
			}
			doDecompression();
		}
		tryDoDecompression();
	} 
  },

  SOSPrepareData : function (underlyingFunction, params){
    var parameters = UTF8ToString(params);
    var uFunction = UTF8ToString(underlyingFunction);
    var array;
    if (parameters.length!==0) {
      array = InternalFunctions[uFunction](JSON.parse(parameters));
    } else {
      array = InternalFunctions[uFunction]();
    }
    BufferStorage.value = array;
    return array.length;
  },

  SOSGetIntValue : function(underlyingFunction, parameters) {
    return AdditionalFunctions[UTF8ToString(underlyingFunction)]();
  },

  SOSGetFloatValue : function(underlyingFunction, parameters) {
    return AdditionalFunctions[UTF8ToString(underlyingFunction)]();
  },

  SOSFillData : function (buffer, type) {
    var array = BufferStorage.value;
    var res;
    if(type == 0) {
      res = new Int32Array(array);
    } else if (type == 1) {
      res = new Float32Array(array);
    } else {
      res = new Uint8Array(array);
    }
    var buffer8Bit = new Uint8Array(res.buffer);
    HEAPU8.set(buffer8Bit, buffer);
  },

  sos_DecodeTexture: function (id, data, length, channels, finalizeTextureCallback) {
    var buffer = new Uint8Array(HEAPU8.buffer, data, length);
    var blob = new Blob( [buffer], { "type": "image/jpeg"});
        
    createImageBitmap(blob, { "colorSpaceConversion": 'none' /*, "imageOrientation": 'flipY'*/ })
    .then( function(image) {
      //var handle = dynCall_iiiii(createTextureCallback, id, image.width, image.height, channels)

      // Get the WebGL texture object from the Emscripten texture ID.
      //textureObj = GL.textures[handle];
      var textureObj = GLctx.createTexture();
      const texId = GL.getNewId(textureObj);
      textureObj.name = texId;
      GL.textures[texId] = textureObj;
  
		// GLctx is the webgl context of the Unity canvas
		GLctx.bindTexture(GLctx.TEXTURE_2D, textureObj);
		GLctx.texParameteri(GLctx.TEXTURE_2D, GLctx.TEXTURE_WRAP_S, GLctx.REPEAT);
		GLctx.texParameteri(GLctx.TEXTURE_2D, GLctx.TEXTURE_WRAP_T, GLctx.REPEAT);
		GLctx.texParameteri(GLctx.TEXTURE_2D, GLctx.TEXTURE_MIN_FILTER, GLctx.LINEAR);
  
		// Upload the canvas image to the GPU texture.
		var internalFormat = channels == 3 ? GLctx.SRGB8 : GLctx.R8;
		var format = channels == 3 ? GLctx.RGB : GLctx.RED;
	    GLctx.texImage2D(GLctx.TEXTURE_2D, 0, internalFormat, format, gl.UNSIGNED_BYTE, image);

		dynCall_viiiii(finalizeTextureCallback, texId, id, image.width, image.height, channels);
    }).catch( function(error){
      console.log(error);
    })
  },

  sos_ReleaseTexture: function (ptr) {
    GLctx.deleteTexture(GL.textures[ptr]);
    GL.textures[ptr] = null;
  }
};

autoAddDeps(SOS_PLUGIN, "$BufferStorage");
autoAddDeps(SOS_PLUGIN, "$ModelStorage");
autoAddDeps(SOS_PLUGIN, "$InternalFunctions");
autoAddDeps(SOS_PLUGIN, "$AdditionalFunctions");
autoAddDeps(SOS_PLUGIN, "$Workers");

mergeInto(LibraryManager.library, SOS_PLUGIN);