﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using UnityEditor;
using UnityEngine;


[ExecuteInEditMode]
public class StreamingModelPreview : MonoBehaviour
{
#if UNITY_EDITOR
    private CancellationTokenSource m_linkedCancellationTokenSource;
    public enum PreviewQuality
    {
        High = 0,
        Normal = 1,
        Low = 2,
        VeryLow = 3,
    }

    public PreviewQuality previewQuality = PreviewQuality.Low;

    private Sos.ScenePreview m_scenePreview;
    private IDestroyTokenWrapper m_destroyTokenWrapper;
    
    private void Awake()
    {
        if(m_destroyTokenWrapper == null) m_destroyTokenWrapper = DestroyTokenWrapperFactory.CreateIDestroyTokenWrapper(this);
    }

    private void Start()
    {
        EditorApplication.wantsToQuit += WantsToQuitCallback;
        EditorApplication.quitting += OnDestroy;
    }
    
    bool WantsToQuitCallback()
    {
        OnDestroy();
        return true;
    }
    private void OnEnable()
    {
        UnityEditor.SceneView.duringSceneGui -= MyOnSceneGUI;
        UnityEditor.SceneView.duringSceneGui += MyOnSceneGUI;

        CheckModel();
    }
    private void OnDisable()
    {
        UnityEditor.SceneView.duringSceneGui -= MyOnSceneGUI;
        m_destroyTokenWrapper?.CancelActiveToken();
        m_scenePreview?.Dispose();
        m_scenePreview = null;
    }

    private void OnDestroy()
    {
        EditorApplication.wantsToQuit -= WantsToQuitCallback;
        EditorApplication.quitting -= OnDestroy;
        UnityEditor.SceneView.duringSceneGui -= MyOnSceneGUI;
        try
        {
            m_destroyTokenWrapper?.CancelActiveToken();
        }
        catch
        {
        }

        m_destroyTokenWrapper?.Dispose();
        m_destroyTokenWrapper= null;
        try
        {
            m_linkedCancellationTokenSource?.Cancel();
        }
        catch
        {
        }

        m_linkedCancellationTokenSource?.Dispose();
        m_linkedCancellationTokenSource = null;
        
        m_scenePreview?.Dispose();
        m_scenePreview = null;
    }

    private void Update()
    {
        if (isActiveAndEnabled && !Application.isPlaying)
        {
            CheckModel();
        }
    }

    private CancellationToken GetCancellationToken(StreamingModel model)
    {
        CancellationToken cancellationToken = default;
        if (m_destroyTokenWrapper == null) //this should be called in awake, and isn't always there
        {
            m_destroyTokenWrapper = DestroyTokenWrapperFactory.CreateIDestroyTokenWrapper(this);
        }
                
        if(model != null && model.DestroyTokenWrapper != null)
        {
            //cancel if this or the model is destroyed
            if(m_linkedCancellationTokenSource == null)
                m_linkedCancellationTokenSource = CancellationTokenSource.CreateLinkedTokenSource(m_destroyTokenWrapper.CancellationToken,model.DestroyTokenWrapper.CancellationToken); // cancel if either monobehaviour is destroyed
            cancellationToken = m_linkedCancellationTokenSource.Token;
        }
        else
        {
            cancellationToken = m_destroyTokenWrapper.CancellationToken;
        }
        return cancellationToken;
    }
    private void CheckModel()
    {
        var model = GetComponent<StreamingModel>();

        if (model == null)
        {
            m_scenePreview?.Dispose();
            m_scenePreview = null;
            return;
        }

        if (m_scenePreview != null && (
                m_scenePreview.url != model.sourceUrl
             || m_scenePreview.qualityLevel != (int)previewQuality
             || m_scenePreview.hashCode != model.hashCode))
        {
            m_scenePreview?.Dispose();
            m_scenePreview = null;
        }
        if (m_scenePreview == null)
        {
            CancellationToken cancellationToken = GetCancellationToken(model);
            m_scenePreview = new Sos.ScenePreview(model.sourceUrl, model.hashCode, model.transform, (int)previewQuality, cancellationToken);
        }
    }

    public void MyOnSceneGUI(UnityEditor.SceneView scenevie)
    {
        if (isActiveAndEnabled && m_scenePreview != null && !Application.isPlaying)
        {
            m_scenePreview.Draw();
        }
    }
#endif
}
